Package: XDE
Title: XDE: a Bayesian hierarchical model for cross-study analysis of
        differential gene expression
Version: 2.58.0
Date: 2020-09-01
Author: R.B. Scharpf, G. Parmigiani, A.B. Nobel, and H. Tjelmeland
Description: Multi-level model for cross-study detection of
        differential gene expression.
Depends: R (>= 2.10.0), Biobase (>= 2.5.5)
Imports: BiocGenerics, genefilter, graphics, grDevices, gtools,
        methods, stats, utils, mvtnorm, RColorBrewer, GeneMeta,
        siggenes
Suggests: MASS, RUnit
Maintainer: Robert Scharpf <rscharpf@jhsph.edu>
Enhances: coda
License: LGPL-2
Collate: functions.R AllClasses.R AllGenerics.R RUpdates.R
        methods-XdeParameter.R methods-matrix.R methods-data.frame.R
        methods-ExpressionSet.R methods-ExpressionSetList.R
        methods-mergeExpressionSet.R methods-Parameters.R
        methods-XdeMcmc.R xde.R zzz.R
LazyLoad: yes
biocViews: Microarray, DifferentialExpression
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/XDE
git_branch: RELEASE_3_23
git_last_commit: 2b99e82
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:28:05 UTC; biocbuild
