Package: Test
Type: Package
Title: Testing R interface for MRF
Version: 1.0.7
Date: 2012-06-10
Author: HT, RS, MF
Maintainer:  <yourfault@somewhere.net>
Description: Testing R interface for MRF - I have tried to modify the clique part (Not working yet)
License: GPL
Packaged: 2012-10-02 18:13:27 UTC; rscharpf
