##setGeneric("avgConcordant",   function(object, ...) standardGeneric("avgConcordant"))
##setGeneric("avgDiff",   function(object, ...) standardGeneric("avgDiff"))
##setGeneric("avgDiscordant",   function(object, ...) standardGeneric("avgDiscordant"))
setGeneric("bayesianEffectSize",   function(object) standardGeneric("bayesianEffectSize"))
setGeneric("bayesianEffectSize<-",   function(object, value) standardGeneric("bayesianEffectSize<-"))
setGeneric("burnin",   function(object) standardGeneric("burnin"))
setGeneric("burnin<-",   function(object, value) standardGeneric("burnin<-"))
setGeneric("calculateBayesianEffectSize",   function(object) standardGeneric("calculateBayesianEffectSize"))
##setGeneric("calculatePosteriorAvg", function(object, ...) standardGeneric("calculatePosteriorAvg"))
setGeneric("diffExpressed", function(object) standardGeneric("diffExpressed"))
setGeneric("directory", function(object) standardGeneric("directory"))
setGeneric("directory<-", function(object, value) standardGeneric("directory<-"))
setGeneric("firstMcmc", function(object) standardGeneric("firstMcmc"))
setGeneric("firstMcmc<-", function(object, value) standardGeneric("firstMcmc<-"))
setGeneric("geneCenter", function(object) standardGeneric("geneCenter"))
setGeneric("goodnessOfFit", function(object, fit, ...) standardGeneric("goodnessOfFit"))
setGeneric("hyperparameters", function(object) standardGeneric("hyperparameters"))
setGeneric("hyperparameters<-", function(object, value) standardGeneric("hyperparameters<-"))
## setGeneric("integrativeCorrelationFilter", function(object, ...) standardGeneric("integrativeCorrelationFilter"))
setGeneric("iterations", function(object) standardGeneric("iterations"))
setGeneric("iterations<-", function(object, value) standardGeneric("iterations<-"))
setGeneric("lastMcmc", function(object) standardGeneric("lastMcmc"))
setGeneric("lastMcmc<-", function(object, value) standardGeneric("lastMcmc<-"))
setGeneric("notes", function(object) standardGeneric("notes"))
setGeneric("nSamples", function(x) standardGeneric("nSamples"))
setGeneric("output",   function(object) standardGeneric("output"))
setGeneric("output<-",   function(object, value) standardGeneric("output<-"))
setGeneric(".pca", function(object, ...) standardGeneric(".pca"))
setGeneric("phenotypeLabel", function(object) standardGeneric("phenotypeLabel"))
setGeneric("phenotypeLabel<-", function(object, value) standardGeneric("phenotypeLabel<-"))
setGeneric("posteriorAvg", function(object) standardGeneric("posteriorAvg"))
setGeneric("posteriorAvg<-", function(object, value) standardGeneric("posteriorAvg<-"))
setGeneric("qqplot", function(x, y, plot.it = TRUE, xlab = deparse(substitute(x)),
                              ylab = deparse(substitute(y)),...) standardGeneric("qqplot"))
setGeneric("seed", function(object) standardGeneric("seed"))
setGeneric("seed<-", function(object, value) standardGeneric("seed<-"))
setGeneric("savedIterations", function(object) standardGeneric("savedIterations"))
setGeneric("showIterations", function(object) standardGeneric("showIterations"))
setGeneric("showIterations<-", function(object, value) standardGeneric("showIterations<-"))
setGeneric(".standardizedDelta", function(object, ...) standardGeneric(".standardizedDelta"))
setGeneric("standardizeSamples", function(object, ...) standardGeneric("standardizeSamples"))
setGeneric("studyCenter", function(object) standardGeneric("studyCenter"))
setGeneric("studyNames", function(object) standardGeneric("studyNames"))
setGeneric("studyNames<-", function(object, value) standardGeneric("studyNames<-"))
setGeneric(".studySplit", function(object, ...) standardGeneric(".studySplit"))
setGeneric("thin", function(x,...) standardGeneric("thin"))
setGeneric("thin<-", function(x, value) standardGeneric("thin<-"))
setGeneric("tuning", function(object) standardGeneric("tuning"))
setGeneric("tuning<-", function(object, value) standardGeneric("tuning<-"))
setGeneric("updates", function(object) standardGeneric("updates"))
setGeneric("updates<-", function(object, value) standardGeneric("updates<-"))
setGeneric("zeroNu", function(object, ...) standardGeneric("zeroNu"))


##setGeneric("getHyperparameters", function(object, G, Q, S, ...) standardGeneric("getHyperparameters"))
setGeneric("Params", function(object, one.delta=FALSE, MRF=FALSE) standardGeneric("Params"))
setGeneric("phenotype", function(object, varLabel) standardGeneric("phenotype"))
setGeneric("expressionVector", function(object) standardGeneric("expressionVector"))
setGeneric("vectorize", function(object) standardGeneric("vectorize"))
